var exports = module.exports = {}

function pretendRequest (credentials, context, cb) {
  var base64 = require('base-64')
  var basicAuth = base64.encode(credentials.username + ':' + credentials.password)
  var endpoint = process.env.API_BASE_URL + '/' + process.env.API_VERSION + '/token'

  context.$http.post(endpoint, '', {
    headers: {
      'Content-Type': 'multipart/form-data',
      'Accept': 'application/vnd.api+json',
      'Authorization': 'Basic ' + basicAuth
    }
  }).then((response) => {
    // success callback
    window.localStorage.token = response.body['token']
    cb({
      authenticated: true
    })
  }, (response) => {
    // error callback
    cb({
      authenticated: false,
      error: response.body.errors[0].title
    })
  })
}

exports.login = function(credentials, context, cb) {
  cb = arguments[arguments.length - 1]
  if (this.getToken()) {
    if (cb) cb(true)
    this.onChange(true)
    return
  }
  pretendRequest(credentials, context, (res) => {
    if (res.authenticated) {
      if (cb) cb(true)
      this.onChange(true)
    } else {
      if (cb) cb(false)
      this.onChange(false)
    }
  })
}

exports.getToken = function () {
  if (window.localStorage.token) {
    var actualEpoch = (new Date()).getTime() / 1000
    if (this.getPayload().exp > actualEpoch) {
      return window.localStorage.token
    } else {
      this.logout()
    }
  }
  return false
}

exports.getPayload = function () {
  if (window.localStorage.token) {
    jwtDecode = require('jwt-decode')
    var decoded = jwtDecode(window.localStorage.token)
    var actualEpoch = (new Date()).getTime() / 1000
    if (decoded.exp > actualEpoch) {
      return decoded
    } else {
      this.logout()
    }
  }
  return false
}

exports.logout = function (cb) {
  delete window.localStorage.token
  if (cb) cb()
  this.onChange(false)
}

exports.loggedIn = function () {
  return (this.getToken() !== false)
}

exports.onChange = function () {}
